#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
This is an example script to perform the sf6-sink correction

For details, please refer to accompanying publication:
    Garny et al, 2023, ACPD: Correction of stratospheric age-of-air derived from SF6 for the effect of chemical sinks

@author: Hella Garny, August 2023
"""


import sf6sinkcorr as sf6
import numpy as np
import matplotlib.pyplot as plt
from matplotlib import cm
from netCDF4 import Dataset


############################################################################################
############# Prepare necessary input for the correction scheme ############################

## step one: read in the SF6-reference timeseries at the ground.
# (here: surface values from the model)
tuvZ_ZM_data      = Dataset("SF6_reference.nc","r")
SF6avg = np.squeeze(tuvZ_ZM_data.variables["SF6ea"][:])      # starts in 1950!
tuvZ_ZM_data.close()

# get the time-dependent function Ft that is needed fot the correction scheme:
Ft = sf6.sf6_ft(SF6avg)


################ coefficients for sink correction ###############
# careful: those values are best estimates based on EMAC model simulations (see Garny et al, 2023, ACPD). 
# Future improvements are desireable.
## model derived coefficients based on full model data and global coefficients:
# exponential fit correction (recommended)
alpha = 5.20991983e+02
beta = -3.80974431e-01
#  3rd order polynomial fit correction (not recommended, see Garny et al, 2023)
CoeffP3 =  ( 536.02004524, -273.30069249,   72.13067853,   -7.32935842)


##########################################################################################
############ get sf6-based aoa data to perform correction on #############################


##  here: read in Example data from EMAC simulations for January 2000
file="SF6_AoA_EMAC_Jan2000.nc"
tuvZ_ZM_data      = Dataset(file,"r")
# grid
lat   = tuvZ_ZM_data.variables["lat"][:]                    # latitudes
plev = tuvZ_ZM_data.variables["lev"][:]
AoA_SF6 = np.squeeze(tuvZ_ZM_data.variables["AoA_SF6"][:,:])
print("shape of AoA:", AoA_SF6.shape)
tuvZ_ZM_data.close()


########## sink correction of  data ######################################################
AoA_corr = np.zeros(AoA_SF6.shape)
AoA_corr_P3 = np.zeros(AoA_SF6.shape)
#get Ft for year 2000
Ft_t = Ft[2000-1950]
#correct for each latitude
for ilat in np.arange(0,AoA_SF6.shape[0]):
    AoA_corr[ilat,:] = sf6.sf6_correct(AoA_SF6[ilat,:],Ft_t,alpha,beta)    
    AoA_corr_P3[ilat,:] = sf6.sf6_correct_poly(AoA_SF6[ilat,:],Ft_t,CoeffP3,(0,5))
    


############## Plot ######################################################################
# 1.) Exponential correction
fig1, ax1 = plt.subplots(1,3, figsize=(15,5))

aoa_clevels = np.arange(0,10,1)
aoa_diff_clevels = (-4, -2, -1.5, -1, -0.9, -0.8, -0.7, -0.6, -0.5, -0.4, -0.3, -0.2, -0.1, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1, 1.5, 2, 4)  #np.arange(-2,2,0.1)

# original sf6 aoa data
csTemp = ax1[0].contourf(lat,plev,np.transpose(AoA_SF6),aoa_clevels,extend="both") #,cmap=cm.seismic)
cbar = fig1.colorbar(csTemp,ax=ax1[0])
cbar.set_label("aoa [yrs]")
ax1[0].set_ylim((0.1, 200))
ax1[0].set_yscale("log")
ax1[0].invert_yaxis()
ax1[0].set_title(f"SF6 AOA EMAC January 2000")
ax1[0].set_xlabel("Latitude $\phi$")
ax1[0].set_ylabel("Pressure [hPa]")

# corrected aoa
csTemp = ax1[1].contourf(lat,plev,np.transpose(AoA_corr),aoa_clevels,extend="both") #,cmap=cm.seismic)
cbar = fig1.colorbar(csTemp,ax=ax1[1])
cbar.set_label(" aoa [yrs]")
ax1[1].set_ylim((0.1, 200))
ax1[1].set_yscale("log")
ax1[1].invert_yaxis()
ax1[1].set_title(f"SF6 AOA corrected Exp")
ax1[1].set_xlabel("Latitude $\phi$")

#difference
csTemp = ax1[2].contourf(lat,plev,np.transpose(AoA_SF6-AoA_corr),aoa_diff_clevels,extend="both",cmap=cm.seismic)
cbar = fig1.colorbar(csTemp,ax=ax1[2])
cbar.ax.set_yticks(aoa_diff_clevels)
cbar.set_label(f"$\Delta$ aoa [yrs]")
ax1[2].set_ylim((0.1, 200))
ax1[2].set_yscale("log")
ax1[2].invert_yaxis()
ax1[2].set_title(f"uncorrected - corrected Exp")
ax1[2].set_xlabel("Latitude $\phi$")


plt.show()


# 2.) 3rd order polynomial correction
fig1, ax1 = plt.subplots(1,3, figsize=(15,5))

aoa_clevels = np.arange(0,10,1)
aoa_diff_clevels = (-4, -2, -1.5, -1, -0.9, -0.8, -0.7, -0.6, -0.5, -0.4, -0.3, -0.2, -0.1, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1, 1.5, 2, 4)  #np.arange(-2,2,0.1)

# original sf6 aoa data
csTemp = ax1[0].contourf(lat,plev,np.transpose(AoA_SF6),aoa_clevels,extend="both") #,cmap=cm.seismic)
cbar = fig1.colorbar(csTemp,ax=ax1[0])
cbar.set_label("aoa [yrs]")
ax1[0].set_ylim((0.1, 200))
ax1[0].set_yscale("log")
ax1[0].invert_yaxis()
ax1[0].set_title(f"SF6 AOA EMAC January 2000")
ax1[0].set_xlabel("Latitude $\phi$")
ax1[0].set_ylabel("Pressure [hPa]")

# corrected aoa
csTemp = ax1[1].contourf(lat,plev,np.transpose(AoA_corr_P3),aoa_clevels,extend="both") #,cmap=cm.seismic)
cbar = fig1.colorbar(csTemp,ax=ax1[1])
cbar.set_label(" aoa [yrs]")
ax1[1].set_ylim((0.1, 200))
ax1[1].set_yscale("log")
ax1[1].invert_yaxis()
ax1[1].set_title(f"SF6 AOA corrected P3")
ax1[1].set_xlabel("Latitude $\phi$")

#difference
csTemp = ax1[2].contourf(lat,plev,np.transpose(AoA_SF6-AoA_corr_P3),aoa_diff_clevels,extend="both",cmap=cm.seismic)
cbar = fig1.colorbar(csTemp,ax=ax1[2])
cbar.set_label(f"$\Delta$ aoa [yrs]")
cbar.ax.set_yticks(aoa_diff_clevels)
ax1[2].set_ylim((0.1, 200))
ax1[2].set_yscale("log")
ax1[2].invert_yaxis()
ax1[2].set_title(f"uncorrected - corrected P3")
ax1[2].set_xlabel("Latitude $\phi$")


plt.show()


